@@ -9,6 +9,7 @@ sys.path.insert(0, os.path.abspath('../src'))
 # mock some things "by hand", so we can import reader below without any dependencies
 for name in [
     'humanize',
+    'readtime',
     'flask',
     'flask.signals',
     'werkzeug',
@@ -45,6 +46,8 @@ autodoc_mock_imports = [
     'humanize',
     'markupsafe',
     'yaml',
+    'jinja2',
+    'tweepy',
 ]
 
 master_doc = 'index'
