@@ -37,7 +37,7 @@ Print the titles of the unread entries::
 
 """
 
-__version__ = '2.14'
+__version__ = '3.0'
 
 from .core import (
     Reader as Reader,
@@ -70,9 +70,6 @@ from .exceptions import (
     EntryError as EntryError,
     EntryExistsError as EntryExistsError,
     EntryNotFoundError as EntryNotFoundError,
-    MetadataError as MetadataError,
-    MetadataNotFoundError as MetadataNotFoundError,
-    FeedMetadataNotFoundError as FeedMetadataNotFoundError,
     StorageError as StorageError,
     SearchError as SearchError,
     SearchNotEnabledError as SearchNotEnabledError,
@@ -82,6 +79,7 @@ from .exceptions import (
     ResourceNotFoundError as ResourceNotFoundError,
     PluginError as PluginError,
     InvalidPluginError as InvalidPluginError,
+    PluginInitError as PluginInitError,
     ReaderWarning as ReaderWarning,
 )
 
