@@ -4,10 +4,14 @@ sqlite3 utilities. Contains no business logic.
 """
 import functools
 import sqlite3
+import sys
+import threading
 import time
 import traceback
+import weakref
 from contextlib import closing
 from contextlib import contextmanager
+from contextlib import nullcontext
 from dataclasses import dataclass
 from datetime import datetime
 from typing import Any
@@ -15,6 +19,7 @@ from typing import Callable
 from typing import cast
 from typing import Dict
 from typing import Iterator
+from typing import List
 from typing import no_type_check
 from typing import Optional
 from typing import Sequence
@@ -157,12 +162,8 @@ def foreign_keys_off(db: sqlite3.Connection) -> Iterator[sqlite3.Connection]:
     [2]: https://sqlite.org/foreignkeys.html#fk_enable
 
     """
-    # TODO: this assert should fail with DBError
     assert not db.in_transaction, "foreign_keys_off must be used outside transactions"
-
-    # TODO: this assignment should fail with DBError
     (foreign_keys,) = db.execute("PRAGMA foreign_keys;").fetchone()
-
     try:
         db.execute("PRAGMA foreign_keys = OFF;")
         yield db
@@ -333,7 +334,6 @@ def table_count(db: sqlite3.Connection) -> int:
 
 def require_version(db: sqlite3.Connection, version_info: Tuple[int, ...]) -> None:
     with closing(db.cursor()) as cursor:
-        # TODO: this assignment should fail with DBError
         ((version,),) = cursor.execute("SELECT sqlite_version();")
 
     version_ints = tuple(int(i) for i in version.split('.'))
@@ -421,6 +421,185 @@ def rowcount_exactly_one(
     assert cursor.rowcount == 1, "shouldn't have more than 1 row"
 
 
+class UsageError(DBError):
+    display_name = "usage error"
+
+
+class LocalConnectionFactory:
+
+    """Maintain a set of connections to the same database, one per thread.
+
+    connect() on object creation in the creating thread,
+    and on the first call in all other threads.
+
+    When close() is called in a thread, run "pragma optimize" and
+    close the connection in that thread.
+
+    If close() is not called, attempt to call close() before the thread ends.
+    This is unreliable: it doesn't work on PyPy,
+    or if the thread was not created through the threading module.
+
+    To account for that and for long-running threads,
+    run optimize regularly (currently, every 1024 calls).
+
+    When used as a context manager, don't run optimize regulary,
+    and call close() automatically when exiting the with block.
+
+    https://github.com/lemon24/reader/issues/206#issuecomment-1183660880
+
+    """
+
+    INLINE_OPTIMIZE_TIMEOUT = 0.1
+
+    def __init__(self, path: str, **kwargs: Any):
+        self.path = path
+        self.kwargs = kwargs
+        if kwargs.get('uri'):  # pragma: no cover
+            raise NotImplementedError("_is_private() does not work for uri=True")
+        self._local = _LocalConnectionFactoryState()
+        self._local.is_creating_thread = True
+        # connect immediately, so exceptions are raised before the first call
+        self.__call__()
+
+    def __call__(self) -> sqlite3.Connection:
+        db = self._local.db
+        if db:
+            if not self._local.context_stack:
+                if self._should_optimize(self._local.call_count):
+                    self._optimize(db, self.INLINE_OPTIMIZE_TIMEOUT)
+                self._local.call_count += 1
+            return db
+
+        if self._is_private(self.path) and not self._local.is_creating_thread:
+            raise UsageError(
+                "cannot use a private database "
+                "from threads other than the creating thread"
+            )
+
+        if self._is_private(self.path) and self._local.closed:
+            raise UsageError("cannot reuse a private database after close()")
+
+        self._local.db = db = sqlite3.connect(self.path, **self.kwargs)
+        self._local.call_count = 0
+
+        # http://threebean.org/blog/atexit-for-threads/
+        # works on cpython (finalizer runs in thread),
+        # but not on pypy (finalizer runs in main thread);
+        # also see https://bugs.python.org/issue14073
+        self._local.finalizer = weakref.finalize(
+            threading.current_thread(), self._close, db
+        )
+
+        return db
+
+    def __enter__(self) -> sqlite3.Connection:
+        self._local.context_stack.append(None)
+        return self.__call__()
+
+    def __exit__(self, *args: Any) -> None:
+        self._local.context_stack.pop()
+        if not self._local.context_stack and not self._is_private(self.path):
+            self.close()
+
+    def close(self) -> None:
+        if self._local.finalizer:
+            self._local.finalizer()
+            self._local.db = None
+            self._local.finalizer = None
+            self._local.call_count = 0
+            self._local.closed = True
+
+    @classmethod
+    def _close(cls, db: sqlite3.Connection) -> None:
+        try:
+            try:
+                cls._optimize(db)
+            finally:
+                db.close()
+        except sqlite3.ProgrammingError as e:
+            message = str(e).lower()
+            # calling close() a second time is a noop
+            if "cannot operate on a closed database" in message:  # pragma: no cover
+                return
+            # can't close() a connection from a thread that didn't create it;
+            # SQLAlchemy ignores this as well in SingletonThreadPool.dispose()
+            if "objects created in a thread" in message:
+                return
+            raise
+
+    @staticmethod
+    def _optimize(db: sqlite3.Connection, timeout: float = 0) -> None:
+        # Don't wait too much for a lock, it means the database is busy,
+        # and now is likely not a good time to run optimize.
+        # Also prevents rare "database is locked" errors in certain conditions
+        # (e.g. test_asyncio_shared on Linux, on Python 3.8 but not later).
+        # https://www2.fossil-scm.org/fossil/artifact/b47bdc17?ln=2556-2559
+
+        # TODO: Once SQLite 3.32 becomes widespread, use "PRAGMA analysis_limit"
+        # to prevent "PRAGMA optimize" from taking too long.
+        # https://github.com/lemon24/reader/issues/143#issuecomment-663433197
+
+        # busy_timeout causes PyPy 7.3.9 to segfault during tests
+        on_pypy = sys.implementation.name == 'pypy'
+        ctx = busy_timeout(db, timeout) if not on_pypy else nullcontext(db)
+
+        try:
+            with ctx:
+                db.execute("PRAGMA optimize;")
+        except sqlite3.OperationalError as e:
+            message = str(e).lower()
+            if "database is locked" not in message:  # pragma: no cover
+                raise
+
+    @staticmethod
+    def _should_optimize(count: int) -> bool:
+        if count > 1024:
+            return count % 2048 == 0
+        return count in {2, 4, 8, 16, 64, 256, 1024}
+
+    @staticmethod
+    def _is_private(path: str) -> bool:
+        """Does connect(path) from another thread connect to a different database?
+
+        With uri=False (the default), only ':memory:' and '' are private.
+
+        With uri=True:
+
+        * file::memory: and file: are private
+        * file::memory:?&cache=shared is shared (one per process)
+        * file:mydb?mode=memory&cache=shared is shared
+        * file:?&cache=shared is ???
+
+        https://www.sqlite.org/c3ref/open.html#urifilenamesinsqlite3open
+        https://www.sqlite.org/uri.html
+
+        """
+        return path in [':memory:', '']
+
+
+class _LocalConnectionFactoryState(threading.local):
+    def __init__(self) -> None:
+        self.db: Optional[sqlite3.Connection] = None
+        self.finalizer: Optional[weakref.finalize] = None
+        self.is_creating_thread: bool = False
+        self.context_stack: List[None] = []
+        self.call_count: int = 0
+        self.closed: bool = False
+
+
+@contextmanager
+def busy_timeout(
+    db: sqlite3.Connection, seconds: float
+) -> Iterator[sqlite3.Connection]:
+    new = int(seconds * 1000)
+    (old,) = db.execute("PRAGMA busy_timeout;").fetchone()
+    db.execute(f"PRAGMA busy_timeout = {new};")
+    try:
+        yield db
+    finally:
+        db.execute(f"PRAGMA busy_timeout = {old};")
+
+
 # BEGIN DebugConnection
 
 # No type annotations or coverage for this;
