@@ -5,7 +5,6 @@ from datetime import datetime
 from datetime import timedelta
 from functools import partial
 from typing import Any
-from typing import cast
 from typing import Dict
 from typing import Iterable
 from typing import Mapping
@@ -22,6 +21,7 @@ from ._sql_utils import BaseQuery
 from ._sql_utils import paginated_query
 from ._sql_utils import Query
 from ._sqlite_utils import DBError
+from ._sqlite_utils import LocalConnectionFactory
 from ._sqlite_utils import rowcount_exactly_one
 from ._sqlite_utils import setup_db as setup_sqlite_db
 from ._sqlite_utils import wrap_exceptions
@@ -241,130 +241,6 @@ def create_indexes(db: sqlite3.Connection) -> None:
     db.execute("CREATE INDEX entries_by_feed ON entries (feed);")
 
 
-def update_from_29_to_30(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # for https://github.com/lemon24/reader/issues/251
-    db.execute("CREATE INDEX entries_by_feed ON entries (feed);")
-
-
-def recreate_search_triggers(db: sqlite3.Connection) -> None:  # pragma: no cover
-    from ._search import Search
-
-    search = Search(db)
-    if search.is_enabled():
-        search._drop_triggers()
-        search._create_triggers()
-
-
-def update_from_31_to_32(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # for https://github.com/lemon24/reader/issues/254
-    db.execute("ALTER TABLE entries ADD COLUMN read_modified TIMESTAMP;")
-    db.execute("ALTER TABLE entries ADD COLUMN important_modified TIMESTAMP;")
-
-
-def update_from_32_to_33(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # https://github.com/lemon24/reader/issues/223
-    db.execute("ALTER TABLE feeds ADD COLUMN subtitle TEXT;")
-    db.execute("ALTER TABLE feeds ADD COLUMN version TEXT;")
-    # force all feeds to update regardless of their caching headers
-    db.execute("UPDATE feeds SET stale = 1;")
-
-
-def update_from_33_to_34(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # for https://github.com/lemon24/reader/issues/239
-
-    # Assumes foreign key checks have already been disabled.
-    # https://sqlite.org/lang_altertable.html#otheralter
-
-    # This migration takes care of:
-    #
-    # * first_updated being added not null
-    # * updated becoming null
-    # * re-creating the entries_by_kinda_* indexes to use first_updated
-    # * added_by being added not null
-
-    create_entries(db, 'new_entries')
-    db.execute(
-        """
-        INSERT INTO new_entries (
-            id,
-            feed,
-            title,
-            link,
-            updated,
-            author,
-            published,
-            summary,
-            content,
-            enclosures,
-            original_feed,
-            data_hash,
-            data_hash_changed,
-            read,
-            read_modified,
-            important,
-            important_modified,
-            added_by,
-            last_updated,
-            first_updated,
-            first_updated_epoch,
-            feed_order
-        )
-        SELECT
-            id,
-            feed,
-            title,
-            link,
-            updated,
-            author,
-            published,
-            summary,
-            content,
-            enclosures,
-            original_feed,
-            data_hash,
-            data_hash_changed,
-            read,
-            read_modified,
-            important,
-            important_modified,
-            'feed',
-            last_updated,
-            first_updated_epoch,
-            first_updated_epoch,
-            feed_order
-        FROM entries;
-        """
-    )
-    # IMPORTANT: this drops ALL indexes and triggers ON entries
-    db.execute("DROP TABLE entries;")
-    db.execute("ALTER TABLE new_entries RENAME TO entries;")
-
-    create_indexes(db)
-    recreate_search_triggers(db)
-
-
-def update_from_34_to_35(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # for https://github.com/lemon24/reader/issues/266
-    db.execute(
-        """
-        INSERT OR REPLACE INTO feed_metadata (feed, key, value)
-        SELECT feed, tag, coalesce((
-            SELECT value FROM feed_metadata AS fm
-            WHERE (fm.feed, fm.key) == (feed, tag)
-        ), 'null')
-        FROM feed_tags;
-        """
-    )
-    db.execute("DROP TABLE feed_tags;")
-    db.execute("ALTER TABLE feed_metadata RENAME TO feed_tags;")
-
-
-def update_from_35_to_36(db: sqlite3.Connection) -> None:  # pragma: no cover
-    # for https://github.com/lemon24/reader/issues/272
-    create_global_tags(db)
-    create_entry_tags(db)
-
-
 # Row value support was added in 3.15.
 # TODO: Remove the Search.update() check once this gets bumped to >=3.18.
 MINIMUM_SQLITE_VERSION = (3, 15)
@@ -381,13 +257,7 @@ def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
             # 1-9 removed before 0.1 (last in e4769d8ba77c61ec1fe2fbe99839e1826c17ace7)
             # 10-16 removed before 1.0 (last in 618f158ebc0034eefb724a55a84937d21c93c1a7)
             # 17-28 removed before 2.0 (last in be9c89581ea491d0c9cc95c9d39f073168a2fd02)
-            29: update_from_29_to_30,
-            30: recreate_search_triggers,
-            31: update_from_31_to_32,
-            32: update_from_32_to_33,
-            33: update_from_33_to_34,
-            34: update_from_34_to_35,
-            35: update_from_35_to_36,
+            # 29-35 removed before 3.0 (last in 69c75529a3f80107b68346d592d6450f9725187c)
         },
         id=APPLICATION_ID,
         minimum_sqlite_version=MINIMUM_SQLITE_VERSION,
@@ -417,6 +287,12 @@ def setup_db(db: sqlite3.Connection, wal_enabled: Optional[bool]) -> None:
 # When adding a new method, add a new test_storage.py::test_errors_locked test.
 
 
+MISSING_MIGRATION_DETAIL = (
+    "; you may have skipped some required migrations, see "
+    "https://reader.readthedocs.io/en/latest/changelog.html#removed-migrations-2-0"
+)
+
+
 class Storage:
 
     # recent_threshold, chunk_size, and entry_counts_average_periods
@@ -443,7 +319,12 @@ class Storage:
             kwargs['factory'] = factory
 
         with wrap_exceptions(StorageError, "error while opening database"):
-            db = self.connect(path, detect_types=sqlite3.PARSE_DECLTYPES, **kwargs)
+            self.factory = LocalConnectionFactory(
+                path,
+                detect_types=sqlite3.PARSE_DECLTYPES,
+                **kwargs,
+            )
+            db = self.factory()
 
         with wrap_exceptions(StorageError, "error while setting up database"):
             try:
@@ -455,39 +336,44 @@ class Storage:
             except DBError as e:
                 message = str(e)
                 if 'no migration' in message:
-                    message += "; you may have skipped some required migrations, see https://reader.readthedocs.io/en/latest/changelog.html#removed-migrations-2-0"
+                    message += MISSING_MIGRATION_DETAIL
                 raise StorageError(message=message) from None
 
-        self.db: sqlite3.Connection = db
         self.path = path
         self.timeout = timeout
 
-    # TODO: these are not part of the Storage API
-    connect = staticmethod(sqlite3.connect)
+    def get_db(self) -> sqlite3.Connection:
+        # Not part of the Storage API to Reader.
+        # Used internally.
+        try:
+            return self.factory()
+        except DBError as e:
+            raise StorageError(message=str(e)) from None
+
+    # Not part of the Storage API to Reader.
+    # Used for testing.
     setup_db = staticmethod(setup_db)
 
     @wrap_exceptions(StorageError)
-    def close(self) -> None:
-        # If "PRAGMA optimize" on every close becomes too expensive, we can
-        # add an option to disable it, or call db.interrupt() after some time.
-        # TODO: Once SQLite 3.32 becomes widespread, use "PRAGMA analysis_limit"
-        # for the same purpose. Details:
-        # https://github.com/lemon24/reader/issues/143#issuecomment-663433197
+    def __enter__(self) -> None:
         try:
-            self.db.execute("PRAGMA optimize;")
-        except sqlite3.ProgrammingError as e:
-            # Calling close() a second time is a noop.
-            if "cannot operate on a closed database" in str(e).lower():
-                return
-            raise
+            self.factory.__enter__()
+        except DBError as e:
+            raise StorageError(message=str(e)) from None
 
-        self.db.close()
+    @wrap_exceptions(StorageError)
+    def __exit__(self, *_: Any) -> None:
+        self.factory.__exit__()
+
+    @wrap_exceptions(StorageError)
+    def close(self) -> None:
+        self.factory.close()
 
     @wrap_exceptions(StorageError)
     def add_feed(self, url: str, added: datetime) -> None:
-        with self.db:
+        with self.get_db() as db:
             try:
-                self.db.execute(
+                db.execute(
                     "INSERT INTO feeds (url, added) VALUES (:url, :added);",
                     dict(url=url, added=added),
                 )
@@ -498,17 +384,15 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def delete_feed(self, url: str) -> None:
-        with self.db:
-            cursor = self.db.execute(
-                "DELETE FROM feeds WHERE url = :url;", dict(url=url)
-            )
+        with self.get_db() as db:
+            cursor = db.execute("DELETE FROM feeds WHERE url = :url;", dict(url=url))
         rowcount_exactly_one(cursor, lambda: FeedNotFoundError(url))
 
     @wrap_exceptions(StorageError)
     def change_feed_url(self, old: str, new: str) -> None:
-        with self.db:
+        with self.get_db() as db:
             try:
-                cursor = self.db.execute(
+                cursor = db.execute(
                     "UPDATE feeds SET url = :new WHERE url = :old;",
                     dict(old=old, new=new),
                 )
@@ -521,7 +405,7 @@ class Storage:
 
             # Some of the fields are not kept from the old feed; details:
             # https://github.com/lemon24/reader/issues/149#issuecomment-700532183
-            self.db.execute(
+            db.execute(
                 """
                 UPDATE feeds
                 SET
@@ -537,7 +421,7 @@ class Storage:
                 (new,),
             )
 
-            self.db.execute(
+            db.execute(
                 """
                 UPDATE entries
                 SET original_feed = (
@@ -580,7 +464,8 @@ class Storage:
             assert False, "shouldn't get here"  # noqa: B011; # pragma: no cover
 
         return zero_or_one(
-            self.db.execute(str(query), dict(url=url)), lambda: FeedNotFoundError(url)
+            self.get_db().execute(str(query), dict(url=url)),
+            lambda: FeedNotFoundError(url),
         )
 
     @wrap_exceptions_iter(StorageError)
@@ -593,7 +478,7 @@ class Storage:
     ) -> Iterable[Tuple[Feed, Optional[_T]]]:
         query, context = make_get_feeds_query(filter_options, sort)
         yield from paginated_query(
-            self.db, query, context, chunk_size, last, feed_factory
+            self.get_db(), query, context, chunk_size, last, feed_factory
         )
 
     @wrap_exceptions(StorageError)
@@ -613,7 +498,7 @@ class Storage:
 
         context = apply_feed_filter_options(query, filter_options)
 
-        row = exactly_one(self.db.execute(str(query), context))
+        row = exactly_one(self.get_db().execute(str(query), context))
 
         return FeedCounts(*row)
 
@@ -650,7 +535,7 @@ class Storage:
             query.scrolling_window_order_by("url")
 
             yield from paginated_query(
-                self.db, query, context, chunk_size, last, FeedForUpdate._make
+                self.get_db(), query, context, chunk_size, last, FeedForUpdate._make
             )
 
         yield from join_paginated_iter(inner, self.chunk_size)
@@ -670,14 +555,14 @@ class Storage:
         #   and can no longer be fetched from.
         rv = []
 
-        with self.db:
+        with self.get_db() as db:
             # We use an explicit transaction for speed,
             # otherwise we get an implicit one for each query).
-            self.db.execute('BEGIN;')
+            db.execute('BEGIN;')
 
             for feed_url, id in entries:  # noqa: B007
                 context = dict(feed_url=feed_url, id=id)
-                row = self.db.execute(
+                row = db.execute(
                     """
                     SELECT
                         updated,
@@ -710,8 +595,8 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def set_feed_user_title(self, url: str, title: Optional[str]) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 "UPDATE feeds SET user_title = :title WHERE url = :url;",
                 dict(url=url, title=title),
             )
@@ -719,8 +604,8 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def set_feed_updates_enabled(self, url: str, enabled: bool) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 "UPDATE feeds SET updates_enabled = :updates_enabled WHERE url = :url;",
                 dict(url=url, updates_enabled=enabled),
             )
@@ -728,8 +613,8 @@ class Storage:
 
     @wrap_exceptions(StorageError)
     def mark_as_stale(self, url: str) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 "UPDATE feeds SET stale = 1 WHERE url = :url;", dict(url=url)
             )
             rowcount_exactly_one(cursor, lambda: FeedNotFoundError(url))
@@ -738,8 +623,8 @@ class Storage:
     def mark_as_read(
         self, feed_url: str, entry_id: str, read: bool, modified: Optional[datetime]
     ) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 """
                 UPDATE entries
                 SET
@@ -761,8 +646,8 @@ class Storage:
         important: bool,
         modified: Optional[datetime],
     ) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 """
                 UPDATE entries
                 SET
@@ -815,8 +700,8 @@ class Storage:
 
         context.update(feed._asdict(), data_hash=feed.hash)
 
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 """
                 UPDATE feeds
                 SET
@@ -840,8 +725,8 @@ class Storage:
         rowcount_exactly_one(cursor, lambda: FeedNotFoundError(intent.url))
 
     def _update_feed_last_updated(self, url: str, last_updated: datetime) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 """
                 UPDATE feeds
                 SET
@@ -856,8 +741,8 @@ class Storage:
     def _update_feed_last_exception(
         self, url: str, last_exception: ExceptionInfo
     ) -> None:
-        with self.db:
-            cursor = self.db.execute(
+        with self.get_db() as db:
+            cursor = db.execute(
                 """
                 UPDATE feeds
                 SET
@@ -961,15 +846,15 @@ class Storage:
             );
         """
 
-        with self.db:
+        with self.get_db() as db:
             try:
                 # we could use executemany(), but it's not noticeably faster
                 for intent in intents:
-                    self.db.execute(query, entry_update_intent_to_dict(intent))
+                    db.execute(query, entry_update_intent_to_dict(intent))
 
             except sqlite3.IntegrityError as e:
                 e_msg = str(e).lower()
-                feed_url, entry_id = intent.entry.object_id
+                feed_url, entry_id = intent.entry.resource_id
 
                 log.debug(
                     "add_entry %r of feed %r: got IntegrityError",
@@ -1008,8 +893,8 @@ class Storage:
         delete_query = "DELETE FROM entries WHERE feed = :feed AND id = :id"
         added_by_query = "SELECT added_by FROM entries WHERE feed = :feed AND id = :id"
 
-        with self.db:
-            cursor = self.db.cursor()
+        with self.get_db() as db:
+            cursor = db.cursor()
 
             for feed_url, entry_id in entries:
                 context = dict(feed=feed_url, id=entry_id)
@@ -1087,7 +972,7 @@ class Storage:
         )
 
         return zero_or_one(
-            self.db.execute(str(query), context),
+            self.get_db().execute(str(query), context),
             lambda: EntryNotFoundError(feed_url, entry_id),
         )
 
@@ -1103,7 +988,7 @@ class Storage:
         query, context = make_get_entries_query(filter_options, sort)
         context.update(recent_threshold=now - self.recent_threshold)
         yield from paginated_query(
-            self.db, query, context, chunk_size, last, entry_factory
+            self.get_db(), query, context, chunk_size, last, entry_factory
         )
 
     @wrap_exceptions(StorageError)
@@ -1121,24 +1006,24 @@ class Storage:
         )
         context.update(new_context)
 
-        row = exactly_one(self.db.execute(str(query), context))
+        row = exactly_one(self.get_db().execute(str(query), context))
 
         return EntryCounts(*row[:4], row[4:7])  # type: ignore[call-arg]
 
     def get_tags(
         self,
-        object_id: AnyResourceId,
+        resource_id: AnyResourceId,
         key: Optional[str] = None,
     ) -> Iterable[Tuple[str, JSONType]]:
         yield from join_paginated_iter(
-            partial(self.get_tags_page, object_id, key),
+            partial(self.get_tags_page, resource_id, key),
             self.chunk_size,
         )
 
     @wrap_exceptions_iter(StorageError)
     def get_tags_page(
         self,
-        object_id: AnyResourceId,
+        resource_id: AnyResourceId,
         key: Optional[str] = None,
         chunk_size: Optional[int] = None,
         last: Optional[_T] = None,
@@ -1146,15 +1031,15 @@ class Storage:
         query = Query().SELECT("key")
         context: Dict[str, Any] = dict()
 
-        if object_id is not None:
-            info = SCHEMA_INFO[len(object_id)]
+        if resource_id is not None:
+            info = SCHEMA_INFO[len(resource_id)]
             query.FROM(f"{info.table_prefix}tags")
 
-            if not any(p is None for p in object_id):
+            if not any(p is None for p in resource_id):
                 query.SELECT("value")
                 for column in info.id_columns:
                     query.WHERE(f"{column} = :{column}")
-                context.update(zip(info.id_columns, object_id))
+                context.update(zip(info.id_columns, resource_id))
             else:
                 query.SELECT_DISTINCT("'null'")
 
@@ -1177,29 +1062,29 @@ class Storage:
             return key, json.loads(value)
 
         yield from paginated_query(
-            self.db, query, context, chunk_size, last, row_factory
+            self.get_db(), query, context, chunk_size, last, row_factory
         )
 
     @overload
-    def set_tag(self, object_id: ResourceId, key: str) -> None:  # pragma: no cover
+    def set_tag(self, resource_id: ResourceId, key: str) -> None:  # pragma: no cover
         ...
 
     @overload
     def set_tag(
-        self, object_id: ResourceId, key: str, value: JSONType
+        self, resource_id: ResourceId, key: str, value: JSONType
     ) -> None:  # pragma: no cover
         ...
 
     @wrap_exceptions(StorageError)
     def set_tag(
         self,
-        object_id: ResourceId,
+        resource_id: ResourceId,
         key: str,
         value: Union[MissingType, JSONType] = MISSING,
     ) -> None:
-        info = SCHEMA_INFO[len(object_id)]
+        info = SCHEMA_INFO[len(resource_id)]
 
-        params = dict(zip(info.id_columns, object_id), key=key)
+        params = dict(zip(info.id_columns, resource_id), key=key)
 
         id_columns = info.id_columns + ('key',)
         id_columns_str = ', '.join(id_columns)
@@ -1228,18 +1113,18 @@ class Storage:
             )
         """
 
-        with self.db:
+        with self.get_db() as db:
             try:
-                self.db.execute(query, params)
+                db.execute(query, params)
             except sqlite3.IntegrityError as e:
                 foreign_key_error = "foreign key constraint failed" in str(e).lower()
                 if not foreign_key_error:  # pragma: no cover
                     raise
-                raise info.not_found_exc(*object_id) from None
+                raise info.not_found_exc(*resource_id) from None
 
     @wrap_exceptions(StorageError)
-    def delete_tag(self, object_id: ResourceId, key: str) -> None:
-        info = SCHEMA_INFO[len(object_id)]
+    def delete_tag(self, resource_id: ResourceId, key: str) -> None:
+        info = SCHEMA_INFO[len(resource_id)]
 
         columns = info.id_columns + ('key',)
         query = f"""
@@ -1250,11 +1135,11 @@ class Storage:
                 {', '.join(('?' for _ in columns))}
             )
         """
-        params = object_id + (key,)
+        params = resource_id + (key,)
 
-        with self.db:
-            cursor = self.db.execute(query, params)
-        rowcount_exactly_one(cursor, lambda: TagNotFoundError(key, cast(str, None)))
+        with self.get_db() as db:
+            cursor = db.execute(query, params)
+        rowcount_exactly_one(cursor, lambda: TagNotFoundError(resource_id, key))
 
 
 def make_get_feeds_query(
