@@ -64,8 +64,8 @@ class FeedData(_namedtuple_compat):
         return Feed(**attrs)
 
     @property
-    def object_id(self) -> str:
-        return self.url
+    def resource_id(self) -> Tuple[str]:
+        return (self.url,)
 
     _hash_exclude_ = frozenset({'url', 'updated'})
 
@@ -121,7 +121,7 @@ class EntryData(_namedtuple_compat):
         return Entry(**attrs)
 
     @property
-    def object_id(self) -> Tuple[str, str]:
+    def resource_id(self) -> Tuple[str, str]:
         return self.feed_url, self.id
 
     _hash_exclude_ = frozenset({'feed_url', 'id', 'updated'})
