@@ -1,4 +1,5 @@
 import functools
+import inspect
 import itertools
 import logging
 import multiprocessing.dummy
@@ -6,7 +7,6 @@ import warnings
 from contextlib import contextmanager
 from functools import wraps
 from queue import Queue
-from textwrap import dedent
 from typing import Any
 from typing import Callable
 from typing import cast
@@ -20,9 +20,6 @@ from typing import TYPE_CHECKING
 from typing import TypeVar
 from typing import Union
 
-from .types import MISSING
-from .types import MissingType
-
 
 FuncType = Callable[..., Any]
 F = TypeVar('F', bound=FuncType)
@@ -31,6 +28,15 @@ _T = TypeVar('_T')
 _U = TypeVar('_U')
 
 
+class MissingType:
+    def __repr__(self) -> str:
+        return "no value"
+
+
+#: Sentinel object used to detect if the `default` argument was provided."""
+MISSING = MissingType()
+
+
 def zero_or_one(
     it: Iterable[_U],
     make_exc: Callable[[], Exception],
@@ -169,7 +175,19 @@ def wrap_map(map: F, workers: int) -> F:
     return cast(F, wrapper)
 
 
-class PrefixLogger(logging.LoggerAdapter):
+# The type of map() should be
+#
+#   Callable[[Callable[[_T], _U], Iterable[_T]], Iterator[_U]]
+#
+# but mypy gets confused; known issue:
+#
+# https://github.com/python/mypy/issues/1317
+# https://github.com/python/mypy/issues/6697
+#
+MapType = Callable[[Callable[[Any], Any], Iterable[Any]], Iterator[Any]]
+
+
+class PrefixLogger(logging.LoggerAdapter):  # type: ignore
 
     # if needed, add: with log.push('another prefix'): ...
 
@@ -185,34 +203,83 @@ class PrefixLogger(logging.LoggerAdapter):
         return ': '.join(tuple(self._escape(p) for p in self.prefixes) + (msg,)), kwargs
 
 
-def deprecated_wrapper(
-    old_name: str, func: F, deprecated_in: str, removed_in: str
+_DEPRECATED_FUNC_WARNING = """\
+{old_name}() is deprecated and will be removed in reader {removed_in}. \
+Use {new_name}() instead.\
+"""
+_DEPRECATED_FUNC_DOCSTRING = """\
+Deprecated alias for :meth:`{new_name}`.
+{doc}
+.. deprecated:: {deprecated_in}
+    This method will be removed in *reader* {removed_in}.
+    Use :meth:`{new_name}` instead.
+
+"""
+
+_DEPRECATED_PROP_WARNING = """\
+{old_name} is deprecated and will be removed in reader {removed_in}. \
+Use {new_name} instead.\
+"""
+_DEPRECATED_PROP_DOCSTRING = """\
+Deprecated variant of :attr:`{new_name}`.
+{doc}
+.. deprecated:: {deprecated_in}
+    This property will be removed in *reader* {removed_in}.
+    Use :attr:`{new_name}` instead.
+
+"""
+
+
+def _deprecated_wrapper(
+    old_name: str,
+    new_name: str,
+    func: F,
+    deprecated_in: str,
+    removed_in: str,
+    doc: str = '',
+    warning_template: str = _DEPRECATED_FUNC_WARNING,
+    docstring_template: str = _DEPRECATED_FUNC_DOCSTRING,
 ) -> F:
+    format_kwargs = dict(locals())
+
     @wraps(func)
     def old_func(*args, **kwargs):  # type: ignore
-        warnings.warn(
-            f"{old_name}() is deprecated "
-            f"and will be removed in reader {removed_in}. "
-            f"Use {func.__name__}() instead.",
-            DeprecationWarning,
-        )
+        warnings.warn(warning_template.format_map(format_kwargs), DeprecationWarning)
         return func(*args, **kwargs)
 
     old_func.__name__ = old_name
-    old_func.__doc__ = dedent(
-        f"""\
-        Deprecated alias for :meth:`{func.__name__}`.
-
-        .. deprecated:: {deprecated_in}
-            This method will be removed in *reader* {removed_in}.
-            Use :meth:`{func.__name__}` instead.
-
-        """
-    )
-
+    old_func.__doc__ = docstring_template.format_map(format_kwargs)
     return cast(F, old_func)
 
 
+def deprecated_wrapper(
+    old_name: str, func: F, deprecated_in: str, removed_in: str
+) -> F:
+    return _deprecated_wrapper(old_name, func.__name__, func, deprecated_in, removed_in)
+
+
+def deprecated(
+    new_name: str, deprecated_in: str, removed_in: str, property: bool = False
+) -> Callable[[F], F]:
+    if not property:
+        kwargs = {}
+    else:
+        kwargs = dict(
+            warning_template=_DEPRECATED_PROP_WARNING,
+            docstring_template=_DEPRECATED_PROP_DOCSTRING,
+        )
+
+    def decorator(func: F) -> F:
+        doc = inspect.getdoc(func) or ''
+        if doc:  # pragma: no cover
+            doc = '\n' + doc + '\n'
+        return _deprecated_wrapper(
+            func.__name__, new_name, func, deprecated_in, removed_in, doc=doc, **kwargs
+        )
+
+    return decorator
+
+
 def _name(thing: object) -> str:
     name = getattr(thing, '__name__', None)
     if name:
@@ -244,8 +311,8 @@ class BetterStrPartial(Partial[_T]):
 
     def __str__(self) -> str:
         name = _name(self.func)
-        parts = [repr(getattr(v, 'object_id', v)) for v in self.args]
+        parts = [repr(getattr(v, 'resource_id', v)) for v in self.args]
         parts.extend(
-            f"{k}={getattr(v, 'object_id', v)!r}" for k, v in self.keywords.items()
+            f"{k}={getattr(v, 'resource_id', v)!r}" for k, v in self.keywords.items()
         )
         return f"{name}({', '.join(parts)})"
