@@ -33,14 +33,13 @@ from ._types import FeedFilterOptions
 from ._types import fix_datetime_tzinfo
 from ._types import NameScheme
 from ._update import Pipeline
-from ._utils import deprecated
 from ._utils import make_pool_map
 from ._utils import zero_or_one
 from .exceptions import EntryNotFoundError
 from .exceptions import FeedExistsError
-from .exceptions import FeedMetadataNotFoundError
 from .exceptions import FeedNotFoundError
 from .exceptions import ParseError
+from .exceptions import PluginInitError
 from .exceptions import SearchNotEnabledError
 from .exceptions import TagNotFoundError
 from .plugins import _load_plugins
@@ -77,6 +76,8 @@ log = logging.getLogger('reader')
 
 _T = TypeVar('_T')
 _U = TypeVar('_U')
+# mypy doesn't seem to support Self yet.
+_TReader = TypeVar('_TReader', bound='Reader')
 
 AfterEntryUpdateHook = Callable[['Reader', EntryData, EntryUpdateStatus], None]
 FeedUpdateHook = Callable[['Reader', str], None]
@@ -187,6 +188,8 @@ def make_reader(
         StorageError: An error occurred while connecting to storage.
         SearchError: An error occurred while enabling/disabling search.
         InvalidPluginError: An invalid plugin name was passed to ``plugins``.
+        PluginInitError: A plugin failed to initialize.
+        PluginError: An ambiguous plugin-related error occurred.
         ReaderError: An ambiguous exception occurred while creating the reader.
 
     .. versionadded:: 1.6
@@ -216,6 +219,10 @@ def make_reader(
         To get the previous behavior (leave search as-is),
         use ``search_enabled=None``.
 
+    .. versionchanged:: 3.0
+        Wrap exceptions raised during plugin initialization
+        in :exc:`PluginInitError` instead of letting them bubble up.
+
     """
 
     # Do as much work as possible before creating the storage.
@@ -260,9 +267,16 @@ def make_reader(
             _called_directly=False,
         )
 
-        # TODO: (maybe) wrap exceptions raised here in a custom exception
+        # TODO: move this logic to reader.plugins
+        # TODO: show the name the user passed, not that of plugin_func
         for plugin_func in plugin_funcs:
-            plugin_func(reader)
+            try:
+                plugin_func(reader)
+            except Exception as e:
+                raise PluginInitError(
+                    "plugin failed to initialze: "
+                    f"{plugin_func.__module__}:{plugin_func.__qualname__}"
+                ) from e
 
     except BaseException:
         storage.close()
@@ -278,30 +292,32 @@ class Reader:
     Persists feed and entry state, provides operations on them,
     and stores configuration.
 
+
     Currently, the following feed types are supported:
 
     * Atom (provided by `feedparser`_)
     * RSS (provided by `feedparser`_)
     * JSON Feed
 
+    Additional sources can be added through :ref:`plugins <plugins>`.
+
     .. _feedparser: https://feedparser.readthedocs.io/en/latest/
 
 
+    In order to perform maintenance tasks and
+    release underlying resources in a predictable manner,
+    the Reader object should be used as a context manager
+    *from each thread* where it is used.
+    For convenience, it is possible to use a Reader object directly;
+    in this case, maintenance tasks may sometimes be performed
+    before arbitrary method calls return.
+
+
     .. important::
 
         Reader objects should be created using :func:`make_reader`; the Reader
         constructor is not stable yet and may change without any notice.
 
-    .. important::
-
-        The :class:`Reader` object is not thread safe;
-        its methods should be called only from the thread that created it.
-
-        To access the same database from multiple threads,
-        create one instance in each thread.
-        If you have a strong use case preventing you to do so,
-        please +1 / comment in :issue:`206`.
-
 
     .. versionadded:: 1.13
         JSON Feed support.
@@ -309,6 +325,22 @@ class Reader:
     .. versionchanged:: 2.10
         Allow passing a `(feed URL,)` 1-tuple anywhere a feed URL can be passed.
 
+    .. versionchanged:: 2.15
+        Allow using Reader objects as context managers.
+
+    .. versionchanged:: 2.15
+        Allow using Reader objects from threads other than the creating thread.
+
+    .. versionchanged:: 2.16
+        Allow using a Reader object from multiple threads directly
+        (do not require it to be used as a context manager anymore).
+
+    .. versionchanged:: 2.16
+        Allow Reader objects to be reused after closing.
+
+    .. versionchanged:: 2.16
+        Allow using a Reader object from multiple asyncio tasks.
+
     """
 
     def __init__(
@@ -343,7 +375,7 @@ class Reader:
         #:
         #:  The only `entry` attributes guaranteed to be present are
         #:  :attr:`~Entry.feed_url`, :attr:`~Entry.id`,
-        #:  and :attr:`~Entry.object_id`;
+        #:  and :attr:`~Entry.resource_id`;
         #:  all other attributes may be missing
         #:  (accessing them may raise :exc:`AttributeError`).
         #:
@@ -413,23 +445,37 @@ class Reader:
                 "constructor is not stable yet and may change without any notice.",
             )
 
+    def __enter__(self: _TReader) -> _TReader:
+        self._storage.__enter__()
+        return self
+
+    def __exit__(self, *_: Any) -> None:
+        self._storage.__exit__()
+
     def close(self) -> None:
         """Close this :class:`Reader`.
 
         Releases any underlying resources associated with the reader.
 
-        The reader becomes unusable from this point forward;
-        a :exc:`ReaderError` will be raised if any other method is called.
+        The reader can be reused after being closed
+        (but you have to call close() again after that).
+
+        close() should be called *from each thread* where the reader is used.
+        Prefer using the reader as a context manager instead.
 
         Raises:
             ReaderError
 
+        .. versionchanged:: 2.16
+            Allow calling close() from any thread.
+
         """
         self._storage.close()
 
     def add_feed(
         self,
         feed: FeedInput,
+        /,
         exist_ok: bool = False,
         *,
         allow_invalid_url: bool = False,
@@ -464,6 +510,9 @@ class Reader:
         .. versionadded:: 2.8
             The ``exist_ok`` argument.
 
+        .. versionchanged:: 3.0
+            The ``feed`` argument is now positional-only.
+
         """
         url = _feed_argument(feed)
         if not allow_invalid_url:
@@ -475,8 +524,8 @@ class Reader:
             if not exist_ok:
                 raise
 
-    def delete_feed(self, feed: FeedInput, missing_ok: bool = False) -> None:
-        """Delete a feed and all of its entries, metadata, and tags.
+    def delete_feed(self, feed: FeedInput, /, missing_ok: bool = False) -> None:
+        """Delete a feed and all of its entries and tags.
 
         Args:
             feed (str or tuple(str) or Feed): The feed URL.
@@ -494,6 +543,9 @@ class Reader:
         .. versionadded:: 2.8
             The ``missing_ok`` argument.
 
+        .. versionchanged:: 3.0
+            The ``feed`` argument is now positional-only.
+
         """
         url = _feed_argument(feed)
         try:
@@ -503,7 +555,7 @@ class Reader:
                 raise
 
     def change_feed_url(
-        self, old: FeedInput, new: FeedInput, *, allow_invalid_url: bool = False
+        self, old: FeedInput, new: FeedInput, /, *, allow_invalid_url: bool = False
     ) -> None:
         """Change the URL of a feed.
 
@@ -517,7 +569,7 @@ class Reader:
         which get set to None).
         All other feed attributes are set to their default values.
 
-        The entries, tags and metadata are preserved.
+        The entries and tags are preserved.
 
         Args:
             old (str or tuple(str) or Feed): The old feed; must exist.
@@ -543,6 +595,9 @@ class Reader:
             To get the previous behavior (no validation),
             use ``allow_invalid_url=True``.
 
+        .. versionchanged:: 3.0
+            The ``old`` and ``new`` arguments are now positional-only.
+
         """
         old_str = _feed_argument(old)
         new_str = _feed_argument(new)
@@ -675,17 +730,23 @@ class Reader:
             yield fix_datetime_tzinfo(rv_feed, 'updated', 'added', 'last_updated')
 
     @overload
-    def get_feed(self, feed: FeedInput) -> Feed:  # pragma: no cover
+    def get_feed(self, feed: FeedInput, /) -> Feed:  # pragma: no cover
         ...
 
     @overload
     def get_feed(
-        self, feed: FeedInput, default: _T
+        self,
+        feed: FeedInput,
+        default: _T,
+        /,
     ) -> Union[Feed, _T]:  # pragma: no cover
         ...
 
     def get_feed(
-        self, feed: FeedInput, default: Union[MissingType, _T] = MISSING
+        self,
+        feed: FeedInput,
+        default: Union[MissingType, _T] = MISSING,
+        /,
     ) -> Union[Feed, _T]:
         """Get a feed.
 
@@ -703,6 +764,9 @@ class Reader:
             FeedNotFoundError
             StorageError
 
+        .. versionchanged:: 3.0
+            The ``feed`` and ``default`` arguments are now positional-only.
+
         """
         return zero_or_one(
             self.get_feeds(feed=feed),
@@ -751,7 +815,7 @@ class Reader:
         )
         return self._storage.get_feed_counts(filter_options)
 
-    def set_feed_user_title(self, feed: FeedInput, title: Optional[str]) -> None:
+    def set_feed_user_title(self, feed: FeedInput, title: Optional[str], /) -> None:
         """Set a user-defined title for a feed.
 
         Args:
@@ -762,11 +826,14 @@ class Reader:
             FeedNotFoundError
             StorageError
 
+        .. versionchanged:: 3.0
+            The ``feed`` and ``title`` arguments are now positional-only.
+
         """
         url = _feed_argument(feed)
         return self._storage.set_feed_user_title(url, title)
 
-    def enable_feed_updates(self, feed: FeedInput) -> None:
+    def enable_feed_updates(self, feed: FeedInput, /) -> None:
         """Enable updates for a feed.
 
         See :meth:`~Reader.update_feeds` for details.
@@ -780,11 +847,14 @@ class Reader:
 
         .. versionadded:: 1.11
 
+        .. versionchanged:: 3.0
+            The ``feed`` argument is now positional-only.
+
         """
         url = _feed_argument(feed)
         self._storage.set_feed_updates_enabled(url, True)
 
-    def disable_feed_updates(self, feed: FeedInput) -> None:
+    def disable_feed_updates(self, feed: FeedInput, /) -> None:
         """Disable updates for a feed.
 
         See :meth:`~Reader.update_feeds` for details.
@@ -798,6 +868,9 @@ class Reader:
 
         .. versionadded:: 1.11
 
+        .. versionchanged:: 3.0
+            The ``feed`` argument is now positional-only.
+
         """
         url = _feed_argument(feed)
         self._storage.set_feed_updates_enabled(url, False)
@@ -967,7 +1040,7 @@ class Reader:
             for hook in self.after_feeds_update_hooks:
                 hook(self)
 
-    def update_feed(self, feed: FeedInput) -> Optional[UpdatedFeed]:
+    def update_feed(self, feed: FeedInput, /) -> Optional[UpdatedFeed]:
         """Update a single feed.
 
         The feed will be updated even if updates are disabled for it.
@@ -996,6 +1069,9 @@ class Reader:
             Update entries whenever their content changes.
             See :meth:`~Reader.update_feeds` for details.
 
+        .. versionchanged:: 3.0
+            The ``feed`` argument is now positional-only.
+
         """
         _, rv = zero_or_one(
             self.update_feeds_iter(
@@ -1128,17 +1204,23 @@ class Reader:
             )
 
     @overload
-    def get_entry(self, entry: EntryInput) -> Entry:  # pragma: no cover
+    def get_entry(self, entry: EntryInput, /) -> Entry:  # pragma: no cover
         ...
 
     @overload
     def get_entry(
-        self, entry: EntryInput, default: _T
+        self,
+        entry: EntryInput,
+        default: _T,
+        /,
     ) -> Union[Entry, _T]:  # pragma: no cover
         ...
 
     def get_entry(
-        self, entry: EntryInput, default: Union[MissingType, _T] = MISSING
+        self,
+        entry: EntryInput,
+        default: Union[MissingType, _T] = MISSING,
+        /,
     ) -> Union[Entry, _T]:
         """Get an entry.
 
@@ -1156,6 +1238,9 @@ class Reader:
             EntryNotFoundError
             StorageError
 
+        .. versionchanged:: 3.0
+            The ``entry`` and ``default`` arguments are now positional-only.
+
         """
         return zero_or_one(
             self.get_entries(entry=entry),
@@ -1208,6 +1293,7 @@ class Reader:
         self,
         entry: EntryInput,
         read: bool,
+        /,
         modified: Union[MissingType, None, datetime] = MISSING,
     ) -> None:
         """Mark an entry as read or unread,
@@ -1229,6 +1315,9 @@ class Reader:
 
         .. versionadded:: 2.2
 
+        .. versionchanged:: 3.0
+            The ``entry`` and ``read`` arguments are now positional-only.
+
         """
         modified_naive: Optional[datetime]
         if isinstance(modified, MissingType):
@@ -1241,7 +1330,7 @@ class Reader:
         feed_url, entry_id = _entry_argument(entry)
         self._storage.mark_as_read(feed_url, entry_id, bool(read), modified_naive)
 
-    def mark_entry_as_read(self, entry: EntryInput) -> None:
+    def mark_entry_as_read(self, entry: EntryInput, /) -> None:
         """Mark an entry as read.
 
         Alias for ``set_entry_read(entry, True)``.
@@ -1256,10 +1345,13 @@ class Reader:
         .. versionadded:: 1.18
             Renamed from :meth:`mark_as_read`.
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
         self.set_entry_read(entry, True)
 
-    def mark_entry_as_unread(self, entry: EntryInput) -> None:
+    def mark_entry_as_unread(self, entry: EntryInput, /) -> None:
         """Mark an entry as unread.
 
         Alias for ``set_entry_read(entry, False)``.
@@ -1274,6 +1366,9 @@ class Reader:
         .. versionadded:: 1.18
             Renamed from :meth:`mark_as_unread`.
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
         return self.set_entry_read(entry, False)
 
@@ -1281,6 +1376,7 @@ class Reader:
         self,
         entry: EntryInput,
         important: bool,
+        /,
         modified: Union[MissingType, None, datetime] = MISSING,
     ) -> None:
         """Mark an entry as important or unimportant,
@@ -1302,6 +1398,9 @@ class Reader:
 
         .. versionadded:: 2.2
 
+        .. versionchanged:: 3.0
+            The ``entry`` and ``important`` arguments are now positional-only.
+
         """
         modified_naive: Optional[datetime]
         if isinstance(modified, MissingType):
@@ -1316,7 +1415,7 @@ class Reader:
             feed_url, entry_id, bool(important), modified_naive
         )
 
-    def mark_entry_as_important(self, entry: EntryInput) -> None:
+    def mark_entry_as_important(self, entry: EntryInput, /) -> None:
         """Mark an entry as important.
 
         Alias for ``set_entry_important(entry, True)``.
@@ -1331,10 +1430,13 @@ class Reader:
         .. versionadded:: 1.18
             Renamed from :meth:`mark_as_important`.
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
         self.set_entry_important(entry, True)
 
-    def mark_entry_as_unimportant(self, entry: EntryInput) -> None:
+    def mark_entry_as_unimportant(self, entry: EntryInput, /) -> None:
         """Mark an entry as unimportant.
 
         Alias for ``set_entry_important(entry, False)``.
@@ -1349,10 +1451,13 @@ class Reader:
         .. versionadded:: 1.18
             Renamed from :meth:`mark_as_unimportant`.
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
         return self.set_entry_important(entry, False)
 
-    def _mark_entry_as_dont_care(self, entry: EntryInput) -> None:
+    def _mark_entry_as_dont_care(self, entry: EntryInput, /) -> None:
         """Mark an entry as read and unimportant at the same time,
         resulting in the same read_modified and important_modified.
 
@@ -1368,7 +1473,7 @@ class Reader:
         self._storage.mark_as_read(feed_url, entry_id, True, modified_naive)
         self._storage.mark_as_important(feed_url, entry_id, False, modified_naive)
 
-    def add_entry(self, entry: Any) -> None:
+    def add_entry(self, entry: Any, /) -> None:
         """Add a new entry to an existing feed.
 
         ``entry`` can be any :class:`Entry`-like object,
@@ -1417,6 +1522,9 @@ class Reader:
 
         .. versionadded:: 2.5
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
 
         # `entry` is of type Union[EntryDataLikeProtocol, EntryDataTypedDict],
@@ -1438,7 +1546,7 @@ class Reader:
         for entry_hook in self.after_entry_update_hooks:
             entry_hook(self, intent.entry, EntryUpdateStatus.NEW)
 
-    def delete_entry(self, entry: EntryInput, missing_ok: bool = False) -> None:
+    def delete_entry(self, entry: EntryInput, /, missing_ok: bool = False) -> None:
         """Delete an entry.
 
         Currently, only entries added by :meth:`~Reader.add_entry`
@@ -1460,6 +1568,9 @@ class Reader:
         .. versionadded:: 2.8
             The ``missing_ok`` argument.
 
+        .. versionchanged:: 3.0
+            The ``entry`` argument is now positional-only.
+
         """
         try:
             self._storage.delete_entries([_entry_argument(entry)], added_by='user')
@@ -1467,128 +1578,6 @@ class Reader:
             if not missing_ok:
                 raise
 
-    @deprecated('get_tags', '2.8', '3.0')
-    def get_feed_metadata(
-        self,
-        feed: FeedInput,
-        key: Optional[str] = None,
-    ) -> Iterable[Tuple[str, JSONType]]:  # pragma: no cover
-        """Get all or some of the metadata for a feed as ``(key, value)`` pairs.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            key (str or None): Only return the metadata for this key.
-
-        Yields:
-            tuple(str, JSONType): ``(key, value)`` pairs, in undefined order.
-            JSONType is whatever :py:func:`json.dumps` accepts.
-
-        Raises:
-            StorageError
-
-        .. versionchanged:: 1.18
-            :meth:`get_feed_metadata` was renamed to :meth:`get_feed_metadata_item`,
-            :meth:`iter_feed_metadata` was renamed to :meth:`get_feed_metadata`.
-
-        .. versionchanged:: 2.0
-            The ``get_feed_metadata(feed, key, default=no value, /)``
-            (positional arguments only)
-            :meth:`get_feed_metadata_item` alias was removed.
-
-        """
-        return self.get_tags(feed, key=key)
-
-    @overload
-    def get_feed_metadata_item(
-        self, feed: FeedInput, key: str
-    ) -> JSONType:  # pragma: no cover
-        ...
-
-    @overload
-    def get_feed_metadata_item(
-        self, feed: FeedInput, key: str, default: _T
-    ) -> Union[JSONType, _T]:  # pragma: no cover
-        ...
-
-    @deprecated('get_tag', '2.8', '3.0')
-    def get_feed_metadata_item(
-        self, feed: FeedInput, key: str, default: Union[MissingType, _T] = MISSING
-    ) -> Union[JSONType, _T]:  # pragma: no cover
-        """Get metadata for a feed.
-
-        Like ``next(iter(reader.get_feed_metadata(feed, key=key)), (None, default))[1]``,
-        but raises a custom exception instead of :exc:`StopIteration`.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            key (str): The key of the metadata to retrieve.
-            default: Returned if given and no metadata exists for `key`.
-
-        Returns:
-            JSONType: The metadata value.
-            JSONType is whatever :py:func:`json.dumps` accepts.
-
-        Raises:
-            FeedMetadataNotFoundError
-            StorageError
-
-        .. versionadded:: 1.18
-            Renamed from :meth:`get_feed_metadata`.
-
-        """
-        try:
-            if isinstance(default, MissingType):
-                return self.get_tag(feed, key)
-            else:
-                return self.get_tag(feed, key, default)
-        except TagNotFoundError as e:
-            raise FeedMetadataNotFoundError(_feed_argument(feed), e.key) from None
-
-    @deprecated('set_tag', '2.8', '3.0')
-    def set_feed_metadata_item(
-        self, feed: FeedInput, key: str, value: JSONType
-    ) -> None:  # pragma: no cover
-        """Set metadata for a feed.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            key (str): The key of the metadata item to set.
-            value (JSONType): The value of the metadata item to set.
-                JSONType is whatever :py:func:`json.dumps` accepts.
-
-        Raises:
-            FeedNotFoundError
-            StorageError
-
-        .. versionadded:: 1.18
-            Renamed from :meth:`set_feed_metadata`.
-
-        """
-        self.set_tag(feed, key, value)
-
-    @deprecated('delete_tag', '2.8', '3.0')
-    def delete_feed_metadata_item(
-        self, feed: FeedInput, key: str
-    ) -> None:  # pragma: no cover
-        """Delete metadata for a feed.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            key (str): The key of the metadata item to delete.
-
-        Raises:
-            FeedMetadataNotFoundError
-            StorageError
-
-        .. versionadded:: 1.18
-            Renamed from :meth:`delete_feed_metadata`.
-
-        """
-        try:
-            self.delete_tag(feed, key)
-        except TagNotFoundError as e:
-            raise FeedMetadataNotFoundError(_feed_argument(feed), e.key) from None
-
     def enable_search(self) -> None:
         """Enable full-text search.
 
@@ -1649,6 +1638,7 @@ class Reader:
     def search_entries(
         self,
         query: str,
+        /,
         *,
         feed: Optional[FeedInput] = None,
         entry: Optional[EntryInput] = None,
@@ -1747,6 +1737,9 @@ class Reader:
         .. versionadded:: 1.12
             The ``limit`` and ``starting_after`` keyword arguments.
 
+        .. versionchanged:: 3.0
+            The ``query`` argument is now positional-only.
+
         """
         filter_options = EntryFilterOptions.from_args(
             feed, entry, read, important, has_enclosures, feed_tags
@@ -1775,6 +1768,7 @@ class Reader:
     def search_entry_counts(
         self,
         query: str,
+        /,
         *,
         feed: Optional[FeedInput] = None,
         entry: Optional[EntryInput] = None,
@@ -1813,6 +1807,9 @@ class Reader:
 
         .. versionadded:: 1.11
 
+        .. versionchanged:: 3.0
+            The ``query`` argument is now positional-only.
+
         """
 
         filter_options = EntryFilterOptions.from_args(
@@ -1821,70 +1818,8 @@ class Reader:
         now = self._now()
         return self._search.search_entry_counts(query, now, filter_options)
 
-    @deprecated('set_tag', '2.8', '3.0')
-    def add_feed_tag(self, feed: FeedInput, tag: str) -> None:  # pragma: no cover
-        """Add a tag to a feed.
-
-        Adding a tag that the feed already has is a no-op.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            tag (str): The tag to add.
-
-        Raises:
-            FeedNotFoundError
-            StorageError
-
-        .. versionadded:: 1.7
-
-        """
-        self.set_tag(feed, tag)
-
-    @deprecated('delete_tag', '2.8', '3.0')
-    def remove_feed_tag(self, feed: FeedInput, tag: str) -> None:  # pragma: no cover
-        """Remove a tag from a feed.
-
-        Removing a tag that the feed does not have is a no-op.
-
-        Args:
-            feed (str or tuple(str) or Feed): The feed URL.
-            tag (str): The tag to remove.
-
-        Raises:
-            StorageError
-
-        .. versionadded:: 1.7
-
-        """
-        self.delete_tag(feed, tag, True)
-
-    @deprecated('get_tag_keys', '2.8', '3.0')
-    def get_feed_tags(
-        self, feed: Optional[FeedInput] = None
-    ) -> Iterable[str]:  # pragma: no cover
-        """Get all or some of the feed tags.
-
-        Args:
-            feed (str or tuple(str) or Feed or None): Only return the tags for this feed.
-
-        Yields:
-            str: The tags, in alphabetical order.
-
-        Raises:
-            StorageError
-
-        .. versionadded:: 1.7
-
-        """
-        return self.get_tag_keys(feed)
-
-    # FIXME: no wildcards allowed in get_tags, update docstring/changelog
-
     def get_tags(
-        self,
-        resource: ResourceInput,
-        *,
-        key: Optional[str] = None,
+        self, resource: ResourceInput, /, *, key: Optional[str] = None
     ) -> Iterable[Tuple[str, JSONType]]:
         """Get all or some tags of a resource as ``(key, value)`` pairs.
 
@@ -1904,7 +1839,7 @@ class Reader:
 
         Args:
             resource: The resource to get tags for.
-            key (str or None): Only return the metadata for this key.
+            key (str or None): Only return the value for this key.
 
         Yields:
             tuple(str, JSONType): ``(key, value)`` pairs, in undefined order.
@@ -1922,14 +1857,14 @@ class Reader:
             Removed support for the ``(None,)`` (any feed) and
             :const:`None` (any resource) wildcard resource values.
 
+        .. versionchanged:: 3.0
+            The ``resource`` argument is now positional-only.
+
         """
         resource_id = _resource_argument(resource)
         return self._storage.get_tags(resource_id, key)
 
-    def get_tag_keys(
-        self,
-        resource: AnyResourceInput = None,
-    ) -> Iterable[str]:
+    def get_tag_keys(self, resource: AnyResourceInput = None, /) -> Iterable[str]:
         """Get the keys of all or some resource tags.
 
         Equivalent to ``sorted(k for k, _ in reader.get_tags(resource))``.
@@ -1963,6 +1898,9 @@ class Reader:
         .. versionchanged:: 2.10
             Support entry and global tags.
 
+        .. versionchanged:: 3.0
+            The ``resource`` argument is now positional-only.
+
         """
         # TODO: efficient implementation
         resource_id: AnyResourceId
@@ -1978,13 +1916,20 @@ class Reader:
 
     @overload
     def get_tag(
-        self, resource: ResourceInput, key: str
+        self,
+        resource: ResourceInput,
+        key: str,
+        /,
     ) -> JSONType:  # pragma: no cover
         ...
 
     @overload
     def get_tag(
-        self, resource: ResourceInput, key: str, default: _T
+        self,
+        resource: ResourceInput,
+        key: str,
+        default: _T,
+        /,
     ) -> Union[JSONType, _T]:  # pragma: no cover
         ...
 
@@ -1993,6 +1938,7 @@ class Reader:
         resource: ResourceInput,
         key: str,
         default: Union[MissingType, _T] = MISSING,
+        /,
     ) -> Union[JSONType, _T]:
         """Get the value of this resource tag.
 
@@ -2019,22 +1965,28 @@ class Reader:
         .. versionchanged:: 2.10
             Support entry and global tags.
 
+        .. versionchanged:: 3.0
+            The ``resource``, ``key``, and ``default`` arguments are now positional-only.
+
         """
         resource_id = _resource_argument(resource)
-        object_id: Any = resource_id if len(resource_id) != 1 else resource_id[0]  # type: ignore
         return zero_or_one(
             (v for _, v in self._storage.get_tags(resource_id, key)),
-            lambda: TagNotFoundError(key, object_id),
+            lambda: TagNotFoundError(resource_id, key),
             default,
         )
 
     @overload
-    def set_tag(self, resource: ResourceInput, key: str) -> None:  # pragma: no cover
+    def set_tag(self, resource: ResourceInput, key: str, /) -> None:  # pragma: no cover
         ...
 
     @overload
     def set_tag(
-        self, resource: ResourceInput, key: str, value: JSONType
+        self,
+        resource: ResourceInput,
+        key: str,
+        value: JSONType,
+        /,
     ) -> None:  # pragma: no cover
         ...
 
@@ -2043,6 +1995,7 @@ class Reader:
         resource: ResourceInput,
         key: str,
         value: Union[JSONType, MissingType] = MISSING,
+        /,
     ) -> None:
         """Set the value of this resource tag.
 
@@ -2066,6 +2019,9 @@ class Reader:
         .. versionchanged:: 2.10
             Support entry and global tags.
 
+        .. versionchanged:: 3.0
+            The ``resource``, ``key``, and ``value`` arguments are now positional-only.
+
         """
         resource_id = _resource_argument(resource)
         if not isinstance(value, MissingType):
@@ -2074,7 +2030,7 @@ class Reader:
             self._storage.set_tag(resource_id, key)
 
     def delete_tag(
-        self, resource: ResourceInput, key: str, missing_ok: bool = False
+        self, resource: ResourceInput, key: str, /, missing_ok: bool = False
     ) -> None:
         """Delete this resource tag.
 
@@ -2096,18 +2052,19 @@ class Reader:
         .. versionchanged:: 2.10
             Support entry and global tags.
 
+        .. versionchanged:: 3.0
+            The ``resource`` and ``key`` arguments are now positional-only.
+
         """
         resource_id = _resource_argument(resource)
-        object_id: Any = resource_id if len(resource_id) != 1 else resource_id[0]  # type: ignore
         try:
             self._storage.delete_tag(resource_id, key)
-        except TagNotFoundError as e:
+        except TagNotFoundError:
             if not missing_ok:
-                e.object_id = object_id
                 raise
 
-    def make_reader_reserved_name(self, key: str) -> str:
-        """Create a *reader*-reserved tag or metadata name.
+    def make_reader_reserved_name(self, key: str, /) -> str:
+        """Create a *reader*-reserved tag name.
         See :ref:`reserved names` for details.
 
         Uses :attr:`~Reader.reserved_name_scheme` to build names of the format::
@@ -2127,17 +2084,22 @@ class Reader:
 
         .. versionadded:: 1.17
 
+        .. versionchanged:: 3.0
+            The ``key`` argument is now positional-only.
+
         """
         return self._reserved_name_scheme.make_reader_name(key)
 
     def make_plugin_reserved_name(
-        self, plugin_name: str, key: Optional[str] = None
+        self,
+        plugin_name: str,
+        key: Optional[str] = None,
+        /,
     ) -> str:
-        """Create a plugin-reserved tag or metadata name.
+        """Create a plugin-reserved tag name.
         See :ref:`reserved names` for details.
 
-        Plugins should use this to generate names
-        for plugin-specific tags and metadata.
+        Plugins should use this to generate names for plugin-specific tags.
 
         Uses :attr:`~Reader.reserved_name_scheme` to build names of the format::
 
@@ -2160,6 +2122,9 @@ class Reader:
 
         .. versionadded:: 1.17
 
+        .. versionchanged:: 3.0
+            The ``plugin_name`` and ``key`` arguments are now positional-only.
+
         """
         return self._reserved_name_scheme.make_plugin_name(plugin_name, key)
 
