@@ -1,7 +1,5 @@
 from functools import cached_property
-from typing import Any
 from typing import Tuple
-from typing import Union
 
 
 class _FancyExceptionBase(Exception):
@@ -72,13 +70,21 @@ class ResourceNotFoundError(ReaderError):
 
     """
 
-    # TODO: object_id: tuple[str, ...] (but FeedError must become tuple[str]!)
+    @property
+    def resource_id(self) -> Tuple[str, ...]:  # pragma: no cover
+        """The `resource_id` of the resource."""
+        raise NotImplementedError
 
 
 class FeedError(ReaderError):
-    """A feed error occurred."""
+    """A feed error occurred.
 
-    def __init__(self, url: str, message: str = '') -> None:
+    .. versionchanged:: 3.0
+        The ``url`` argument is now positional-only.
+
+    """
+
+    def __init__(self, url: str, /, message: str = '') -> None:
         super().__init__(message)
 
         #: The feed URL.
@@ -89,13 +95,13 @@ class FeedError(ReaderError):
         return repr(self.url)
 
     @property
-    def object_id(self) -> str:
-        """Alias for :attr:`~FeedError.url`.
+    def resource_id(self) -> Tuple[str]:
+        """Alias for (:attr:`~url`,).
 
-        .. versionadded:: 1.12
+        .. versionadded:: 2.17
 
         """
-        return self.url
+        return (self.url,)
 
 
 class FeedExistsError(FeedError):
@@ -133,9 +139,13 @@ class EntryError(ReaderError):
 
     .. versionchanged:: 1.18
         The ``url`` argument/attribute was renamed to ``feed_url``.
+
+    .. versionchanged:: 3.0
+        The ``feed_url`` and ``id`` arguments are now positional-only.
+
     """
 
-    def __init__(self, feed_url: str, id: str, message: str = '') -> None:
+    def __init__(self, feed_url: str, id: str, /, message: str = '') -> None:
         super().__init__(message)
 
         #: The feed URL.
@@ -149,10 +159,10 @@ class EntryError(ReaderError):
         return repr((self.feed_url, self.id))
 
     @property
-    def object_id(self) -> Tuple[str, str]:
-        """Alias for (:attr:`~EntryError.feed_url`, :attr:`~EntryError.id`).
+    def resource_id(self) -> Tuple[str, str]:
+        """Alias for (:attr:`~feed_url`, :attr:`~id`).
 
-        .. versionadded:: 1.12
+        .. versionadded:: 2.17
 
         """
         return self.feed_url, self.id
@@ -174,74 +184,6 @@ class EntryNotFoundError(EntryError, ResourceNotFoundError):
     message = "no such entry"
 
 
-class MetadataError(ReaderError):
-    """A metadata error occurred.
-
-    .. versionchanged:: 1.18
-
-        Signature changed from ``MetadataError(message='')``
-        to ``MetadataError(key, message='')``.
-
-    .. deprecated:: 2.8
-        Will be removed in *reader* 3.0.
-
-    """
-
-    def __init__(self, *args: Any, key: str, **kwargs: Any) -> None:
-        super().__init__(*args, **kwargs)
-
-        #: The metadata key.
-        self.key = key
-
-    @property
-    def _str(self) -> str:
-        return f"{super()._str}: {self.key!r}"
-
-
-class MetadataNotFoundError(MetadataError):
-    """Metadata not found.
-
-    .. versionchanged:: 1.18
-
-        Signature changed from ``MetadataNotFoundError(url, key, message='')``
-        to ``MetadataNotFoundError(key, message='')``.
-
-    .. deprecated:: 2.8
-        Will be removed in *reader* 3.0.
-
-    """
-
-    message = "no such metadata"
-
-
-class FeedMetadataNotFoundError(MetadataNotFoundError, FeedError):
-    """Feed metadata not found.
-
-    .. versionadded:: 1.18
-
-    .. deprecated:: 2.8
-        Will be removed in *reader* 3.0.
-
-    """
-
-    def __init__(self, url: str, key: str, message: str = '') -> None:
-        super().__init__(url, key=key, message=message)
-
-
-class EntryMetadataNotFoundError(MetadataNotFoundError, EntryError):
-    """Entry metadata not found.
-
-    .. versionadded:: 1.18
-
-    .. deprecated:: 2.8
-        Will be removed in *reader* 3.0.
-
-    """
-
-    def __init__(self, feed_url: str, id: str, key: str, message: str = '') -> None:
-        super().__init__(feed_url, id, key=key, message=message)
-
-
 class StorageError(ReaderError):
     """An exception was raised by the underlying storage.
 
@@ -281,32 +223,49 @@ class InvalidPluginError(PluginError, ValueError):
     """
 
 
+class PluginInitError(PluginError):
+    """A plugin failed to initialize.
+
+    The original exception should be chained to this one (e.__cause__).
+
+    .. versionadded:: 3.0
+
+    """
+
+
 class TagError(ReaderError):
     """A tag error occurred.
 
     .. versionadded:: 2.8
 
+    .. versionchanged:: 2.17
+        Signature changed from ``TagError(key, object_id, ...)``
+        to ``TagError(key, resource_id, ...)``.
+
+    .. versionchanged:: 3.0
+        Signature changed from ``TagError(key, resource_id, ...)``
+        to ``TagError(resource_id, key, ...)``.
+
+    .. versionchanged:: 3.0
+        The ``resource_id`` and ``key`` arguments are now positional-only.
+
     """
 
     def __init__(
-        self,
-        key: str,
-        object_id: Union[Tuple[()], str, Tuple[str, str]],
-        message: str = '',
+        self, resource_id: Tuple[str, ...], key: str, /, message: str = ''
     ) -> None:
         super().__init__(message)
 
+        #: The `resource_id` of the resource.
+        self.resource_id = resource_id
+
         #: The tag key.
         self.key = key
 
-        # TODO: tuple[str, ...], once FeedError.object_id becomes tuple[str]
-
-        #: The `object_id` of the resource.
-        self.object_id = object_id
-
     @property
     def _str(self) -> str:
-        return f"{self.object_id!r}: {self.key!r}"
+        parts = self.resource_id + (self.key,)
+        return ': '.join(repr(part) for part in parts)
 
 
 class TagNotFoundError(TagError):
