@@ -26,6 +26,11 @@ from typing import Union
 
 from reader.exceptions import ReaderError
 
+# noreorder
+# can't be defined here because of circular imports
+from reader._utils import MISSING as MISSING  # noqa: F401
+from reader._utils import MissingType as MissingType  # noqa: F401
+
 
 _T = TypeVar('_T')
 
@@ -144,13 +149,13 @@ class Feed(_namedtuple_compat):
     updates_enabled: bool = True
 
     @property
-    def object_id(self) -> str:
-        """Alias for :attr:`~Feed.url`.
+    def resource_id(self) -> Tuple[str]:
+        """Alias for (:attr:`~url`,).
 
-        .. versionadded:: 1.12
+        .. versionadded:: 2.17
 
         """
-        return self.url
+        return (self.url,)
 
 
 _EI = TypeVar('_EI', bound='ExceptionInfo')
@@ -308,10 +313,10 @@ class Entry(_namedtuple_compat):
     feed: Feed = cast(Feed, None)
 
     @property
-    def object_id(self) -> Tuple[str, str]:
-        """Alias for (:attr:`~Entry.feed_url`, :attr:`~Entry.id`).
+    def resource_id(self) -> Tuple[str, str]:
+        """Alias for (:attr:`~feed_url`, :attr:`~id`).
 
-        .. versionadded:: 1.12
+        .. versionadded:: 2.17
 
         """
         return self.feed_url, self.id
@@ -611,10 +616,10 @@ class EntrySearchResult(_namedtuple_compat):
     # TODO: entry: Optional[Entry]; model it through typing if possible
 
     @property
-    def object_id(self) -> Tuple[str, str]:
-        """Alias for (:attr:`~EntrySearchResult.feed_url`, :attr:`~EntrySearchResult.id`).
+    def resource_id(self) -> Tuple[str, str]:
+        """Alias for (:attr:`~feed_url`, :attr:`~id`).
 
-        .. versionadded:: 1.12
+        .. versionadded:: 2.17
 
         """
         return self.feed_url, self.id
@@ -762,15 +767,6 @@ TagFilterInput = Union[
 ]
 
 
-class MissingType:
-    def __repr__(self) -> str:
-        return "no value"
-
-
-#: Sentinel object used to detect if the `default` argument was provided."""
-MISSING = MissingType()
-
-
 @dataclass(frozen=True)
 class FeedCounts(_namedtuple_compat):
 
