@@ -51,12 +51,9 @@ def apply_flaky_pypy_sqlite3(items):  # pragma: no cover
 
 def pytest_runtest_setup(item):
     # lxml fails to build in various places,
-    # see the comments in setup.py for details.
+    # see the comments in setup.cfg for details.
     for mark in item.iter_markers(name="requires_lxml"):
-        no_lxml = [
-            sys.implementation.name == 'pypy',
-            os.name == 'nt' and sys.version_info[:2] == (3, 10),
-        ]
+        no_lxml = [sys.implementation.name == 'pypy' and os.name == 'nt']
         if any(no_lxml):
             pytest.skip("test requires lxml")
 
