@@ -6,7 +6,7 @@ from reader._types import EntryData
 from reader._types import FeedData
 
 
-feed = FeedData(url='{}empty.atom'.format(url_base))
+feed = FeedData(url='{}empty.atom'.format(url_base), version='atom10')
 
 entries = [
     EntryData(
