@@ -1,8 +1,12 @@
 import threading
 from collections import OrderedDict
+from contextlib import nullcontext
 from datetime import timezone
+from io import BytesIO
 
+import reader._parser
 from reader import ParseError
+from reader._parser import RetrieveResult
 from reader._types import EntryData
 from reader._types import FeedData
 from reader._types import fix_datetime_tzinfo
@@ -57,6 +61,17 @@ class Parser:
         return entry
 
     def __call__(self, url, http_etag, http_last_modified):
+        raise NotImplementedError
+
+    parallel = reader._parser.Parser.parallel
+    persistent_session = staticmethod(nullcontext)
+
+    def retrieve(self, url, http_etag, http_last_modified, is_parallel):
+        return nullcontext(RetrieveResult(BytesIO(b'opaque')))
+
+    def parse(self, url, result):
+        assert result.file.read() == b'opaque', result
+
         for feed_number, feed in self.feeds.items():
             if feed.url == url:
                 break
@@ -80,6 +95,12 @@ class Parser:
     def validate_url(self, url):
         pass
 
+    def process_feed_for_update(self, feed):
+        return feed
+
+    def process_entry_pairs(self, url, mime_type, pairs):
+        return pairs
+
 
 class BlockingParser(Parser):
     def __init__(self, *args, **kwargs):
@@ -87,10 +108,13 @@ class BlockingParser(Parser):
         self.in_parser = threading.Event()
         self.can_return_from_parser = threading.Event()
 
-    def __call__(self, *args, **kwargs):
+    def wait(self):
         self.in_parser.set()
         self.can_return_from_parser.wait()
-        return super().__call__(*args, **kwargs)
+
+    def retrieve(self, *args):
+        self.wait()
+        return super().retrieve(*args)
 
 
 class FailingParser(Parser):
@@ -99,19 +123,22 @@ class FailingParser(Parser):
         self.condition = condition
         self.exception = Exception('failing')
 
-    def __call__(self, url, http_etag, http_last_modified):
+    def raise_exc(self, url):
         if self.condition(url):
             try:
                 # We raise so the exception has a traceback set.
                 raise self.exception
             except Exception as e:
                 raise ParseError(url) from e
-        return super().__call__(url, http_etag, http_last_modified)
+
+    def retrieve(self, url, *args):
+        self.raise_exc(url)
+        return super().retrieve(url, *args)
 
 
 class NotModifiedParser(Parser):
-    def __call__(self, *args, **kwargs):
-        return None
+    def retrieve(self, *args):
+        return nullcontext(None)
 
 
 class ParserThatRemembers(Parser):
@@ -119,6 +146,8 @@ class ParserThatRemembers(Parser):
         super().__init__(*args, **kwargs)
         self.calls = []
 
-    def __call__(self, url, http_etag, http_last_modified):
-        self.calls.append((url, http_etag, http_last_modified))
-        return super().__call__(url, http_etag, http_last_modified)
+    def retrieve(self, *args):
+        self.calls.append(args[:3])
+        return super().retrieve(*args)
+
+    # FIXME: remember parse() as well?
