@@ -74,26 +74,3 @@ def test_regex_mark_as_read_bad_metadata(make_reader, value):
     reader.update_feeds()
 
     assert [e.read for e in reader.get_entries()] == [False]
-
-
-@pytest.mark.parametrize('with_entry', [False, True])
-def test_regex_mark_as_read_pre_2_7_metadata(make_reader, with_entry):
-    reader = make_reader(':memory:', plugins=['reader.mark_as_read'])
-
-    parser = Parser()
-    reader._parser = parser
-
-    one = parser.feed(1, datetime(2010, 1, 1))
-    if with_entry:
-        parser.entry(1, 1, datetime(2010, 1, 1), title='match old')
-
-    reader.add_feed(one)
-    reader.set_tag(one, '.reader.mark_as_read', {'title': ['^match']})
-
-    reader.update_feeds()
-
-    assert all(e.read for e in reader.get_entries())
-
-    tags = dict(reader.get_tags(one))
-    assert '.reader.mark_as_read' not in tags
-    assert tags['.reader.mark-as-read'] == {'title': ['^match']}
