@@ -23,15 +23,6 @@ from reader._search import Search
 from reader._storage import Storage
 
 
-def test_closed(reader):
-    reader.close()
-    # TODO: Maybe parametrize with all the methods
-    # (also, unify with test_reader.py::test_closed)
-    with pytest.raises(SearchError) as excinfo:
-        reader.is_search_enabled()
-    assert 'closed' in excinfo.value.message
-
-
 @pytest.fixture(params=[False, True], ids=['without_entries', 'with_entries'])
 def reader_without_and_with_entries(request, make_reader):
     reader = make_reader(':memory:', search_enabled=None)
@@ -457,7 +448,7 @@ def test_update_triggers_no_change(db_path, make_reader, monkeypatch, set_user_t
     # never fires at the moment).
     #
     # Meanwhile, we do a (more intrusive/brittle) manual update:
-    with reader._search.db as db:
+    with reader._search.get_db() as db:
         db.execute(
             """
             UPDATE entries
@@ -911,7 +902,7 @@ def test_update_search_entry_changed_during_strip_html(
     try:
         feed = parser.feed(1, datetime(2010, 1, 3), title='three')
         parser.entry(1, 1, datetime(2010, 1, 3), title='three')
-        reader._storage.db.execute("PRAGMA busy_timeout = 0;")
+        reader._storage.get_db().execute("PRAGMA busy_timeout = 0;")
         reader.update_feed(feed.url)
         expected_title = 'three'
     except StorageError:
@@ -1004,7 +995,9 @@ def test_update_search_entry_changed_between_insert_loops(
     (result,) = reader.search_entries('entry')
     assert len(result.content) == 1
 
-    ((rowcount,),) = reader._search.db.execute("select count(*) from entries_search;")
+    ((rowcount,),) = reader._search.get_db().execute(
+        "select count(*) from entries_search;"
+    )
     assert rowcount == 1
 
 
@@ -1064,7 +1057,9 @@ def test_update_search_concurrent_calls(db_path, make_reader, monkeypatch):
     (result,) = reader.search_entries('entry')
     assert len(result.content) == 2
 
-    ((rowcount,),) = reader._search.db.execute("select count(*) from entries_search;")
+    ((rowcount,),) = reader._search.get_db().execute(
+        "select count(*) from entries_search;"
+    )
     assert rowcount == 2
 
 
@@ -1096,11 +1091,11 @@ def test_add_entry_basic(reader):
     reader.update_search()
 
     (result,) = reader.search_entries('entry')
-    assert result.object_id == ('1', '1')
+    assert result.resource_id == ('1', '1')
     assert result.metadata['.title'].apply('*', '*') == 'my *entry*'
     assert result.content['.summary'].apply('*', '*') == 'I am a summary'
 
     (result,) = reader.search_entries('summary')
-    assert result.object_id == ('1', '1')
+    assert result.resource_id == ('1', '1')
     assert result.metadata['.title'].apply('*', '*') == 'my entry'
     assert result.content['.summary'].apply('*', '*') == 'I am a *summary*'
