@@ -17,18 +17,6 @@ from reader._sqlite_utils import DBError
 from reader._sqlite_utils import require_version
 
 
-def test_bs4_import_error(storage, monkeypatch):
-    search = Search(storage)
-    search.enable()
-    monkeypatch.setattr('reader._search.bs4', None)
-    monkeypatch.setattr('reader._search.bs4_import_error', ImportError('reason'))
-
-    with pytest.raises(SearchError) as excinfo:
-        search.update()
-    assert 'search dependencies' in str(excinfo.value.message)
-    assert isinstance(excinfo.value.__cause__, ImportError)
-
-
 STRIP_HTML_DATA = [(i, i) for i in [None, 10, 11.2, b'aabb', b'aa<br>bb']] + [
     ('aabb', 'aabb'),
     ('aa<br>bb', 'aa\nbb'),
@@ -74,10 +62,7 @@ STRIP_HTML_DATA = [(i, i) for i in [None, 10, 11.2, b'aabb', b'aa<br>bb']] + [
     'features',
     [
         None,
-        # lxml fails to build on pypy, see the comments in setup.py for details.
-        pytest.param(
-            'lxml', marks=pytest.mark.skipif("sys.implementation.name == 'pypy'")
-        ),
+        pytest.param('lxml', marks=pytest.mark.requires_lxml),
         'html.parser',
         'html5lib',
     ],
@@ -132,7 +117,7 @@ def search_entries_chunk_size_1(storage, _, __):
 
 
 def search_entry_counts(storage, _, __):
-    Search(storage).search_entry_counts('entry')
+    Search(storage).search_entry_counts('entry', now=datetime(2010, 1, 1))
 
 
 def search_entry_last(storage, feed, entry):
@@ -226,7 +211,7 @@ def call_search_entries(search, query):
 
 
 def call_search_entry_counts(search, query):
-    search.search_entry_counts(query)
+    search.search_entry_counts(query, datetime(2010, 1, 1))
     raise ActuallyOK
 
 
