@@ -13,7 +13,6 @@ import reader._sqlite_utils
 from reader import EntryNotFoundError
 from reader import FeedNotFoundError
 from reader import InvalidSearchQueryError
-from reader import MetadataNotFoundError
 from reader import StorageError
 from reader._sqlite_utils import DBError
 from reader._sqlite_utils import HeavyMigration
@@ -116,35 +115,39 @@ def test_path(db_path):
     assert storage.path == db_path
 
 
-def test_timeout(db_path):
+def test_timeout(db_path, monkeypatch):
     """Storage.__init__ must pass timeout= to connect."""
 
-    expected_timeout = None
+    original_connect = sqlite3.connect
 
-    class MyStorage(Storage):
-        @classmethod
-        def connect(cls, *args, **kwargs):
-            nonlocal expected_timeout
-            expected_timeout = kwargs.get('timeout')
-            return super().connect(*args, **kwargs)
+    def connect(*args, **kwargs):
+        connect.expected_timeout = kwargs.get('timeout')
+        return original_connect(*args, **kwargs)
 
-    MyStorage(db_path, 19)
+    monkeypatch.setattr('sqlite3.connect', connect)
+    Storage(db_path, 19)
 
-    assert expected_timeout == 19
+    assert connect.expected_timeout == 19
 
 
 def test_close():
-    storage = Storage(':memory:')
+    close_called = False
 
-    storage.db.execute('values (1)')
+    class Connection(sqlite3.Connection):
+        def close(self):
+            super().close()
+            nonlocal close_called
+            close_called = True
+
+    storage = Storage(':memory:', factory=Connection)
+    storage.get_db().execute('values (1)')
 
     storage.close()
+    assert close_called
+
     # no-op a second time
     storage.close()
 
-    with pytest.raises(sqlite3.ProgrammingError):
-        storage.db.execute('values (1)')
-
 
 def test_close_error():
     class Connection(sqlite3.Connection):
@@ -154,7 +157,7 @@ def test_close_error():
         raise sqlite3.ProgrammingError('unexpected error')
 
     storage = Storage(':memory:', factory=Connection)
-    storage.db.execute = execute
+    storage.get_db().execute = execute
 
     # should not be StorageError, because it's likely a bug
     with pytest.raises(sqlite3.ProgrammingError):
@@ -243,7 +246,7 @@ def add_entry(storage, feed, entry):
 
 
 def delete_entries(storage, feed, entry):
-    storage.delete_entries([entry.object_id])
+    storage.delete_entries([entry.resource_id])
 
 
 def get_entries_chunk_size_0(storage, _, __):
@@ -325,7 +328,7 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
 
     # WAL provides more concurrency; some things won't to block with it enabled.
     storage = Storage(db_path, wal_enabled=False)
-    storage.db.execute("PRAGMA busy_timeout = 0;")
+    storage.get_db().execute("PRAGMA busy_timeout = 0;")
 
     feed = FeedData('one')
     entry = EntryData('one', 'entry', datetime(2010, 1, 2))
@@ -341,11 +344,12 @@ def check_errors_locked(db_path, pre_stuff, do_stuff, exc_type):
 
     def target():
         storage = Storage(db_path, wal_enabled=False)
-        storage.db.isolation_level = None
-        storage.db.execute("BEGIN EXCLUSIVE;")
+        db = storage.get_db()
+        db.isolation_level = None
+        db.execute("BEGIN EXCLUSIVE;")
         in_transaction.set()
         can_return_from_transaction.wait()
-        storage.db.execute("ROLLBACK;")
+        db.execute("ROLLBACK;")
 
     if pre_stuff:
         pre_stuff(storage, feed, entry)
@@ -577,9 +581,9 @@ def test_migration_minimum_version(db_path, request):
     storage = Storage(db_path)
     request.addfinalizer(storage.close)
 
-    assert HeavyMigration.get_version(storage.db) >= MIGRATION_MINIMUM_VERSION
+    assert HeavyMigration.get_version(storage.get_db()) >= MIGRATION_MINIMUM_VERSION
 
-    HeavyMigration.set_version(storage.db, MIGRATION_MINIMUM_VERSION - 1)
+    HeavyMigration.set_version(storage.get_db(), MIGRATION_MINIMUM_VERSION - 1)
 
     with pytest.raises(StorageError) as excinfo:
         Storage(db_path)
